@@ -30,6 +30,7 @@
 #include <iterator>
 #include <functional>
 #include <stack>
+#include <unordered_set>
 
 
 namespace GiNaC {
@@ -69,7 +70,10 @@ class scalar_products;
 class const_iterator;
 class const_preorder_iterator;
 class const_postorder_iterator;
-
+class symbol;
+struct symbolhasher;
+using symbolset = std::unordered_set<symbol,symbolhasher>;
+using expairvec = std::vector<std::pair<ex,ex>>;
 
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
@@ -83,9 +87,9 @@ class ex {
 	template<class T> friend inline bool is_a(const ex &);
 	template<class T> friend inline bool is_exactly_a(const ex &);
 	
-	friend struct print_order;
-	friend struct print_order_mul;
-	friend struct print_order_pair;
+	friend class print_order;
+	friend class print_order_mul;
+	friend class print_order_pair;
 	// default constructor, copy constructor and assignment operator
 public:
 	ex() throw();
@@ -127,9 +131,8 @@ public:
 
 	// evaluation
 	ex eval(int level = 0) const { return bp->eval(level); }
-	ex evalf(int level = 0, PyObject* parent=NULL) const 
+	ex evalf(int level = 0, PyObject* parent=nullptr) const 
 	{ return bp->evalf(level, parent); }
-	ex evalm() const { return bp->evalm(); }
 	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
 	ex eval_integ() const { return bp->eval_integ(); }
 
@@ -144,7 +147,9 @@ public:
 	// operand access
 	size_t nops() const { return bp->nops(); }
 	size_t nsymbols() const;
-	ex op(size_t i) const { return bp->op(i); }
+        bool get_first_symbol(ex &x) const;
+        symbolset symbols() const;
+	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
 	ex operator[](size_t i) const { return (*bp)[i]; }
@@ -187,6 +192,7 @@ public:
 	ex coeff(const ex & s, int n = 1) const { return bp->coeff(s, n); }
 	ex lcoeff(const ex & s) const { return coeff(s, degree(s)); }
 	ex tcoeff(const ex & s) const { return coeff(s, ldegree(s)); }
+        void coefficients(const ex & s, expairvec & vec) const;
 
 	// expand/collect
 	ex expand(unsigned options=0) const;
@@ -195,16 +201,22 @@ public:
 	// differentiation and series expansion
 	ex diff(const symbol & s, unsigned nth = 1) const;
 	ex series(const ex & r, int order, unsigned options = 0) const;
+        void useries(flint_series_t& fp, int order) const { return bp->useries(fp, order); }
 
 	// rational functions
-	ex normal(int level = 0) const;
+	ex normal(int level = 0, bool noexpand_combined=false,
+                        bool noexpand_numer=true) const;
 	ex to_rational(exmap & repl) const;
 	ex to_rational(lst & repl_lst) const;
 	ex to_polynomial(exmap & repl) const;
 	ex to_polynomial(lst & repl_lst) const;
+#ifdef PYNAC_HAVE_LIBGIAC
+        const giac::polynome to_polynome(ex_int_map& map, exvector& revmap) const;
+#endif
 	ex numer() const;
 	ex denom() const;
 	ex numer_denom() const;
+        ex combine_fractions(bool deep=true) const;
 
 	// polynomial algorithms
 	ex unit(const ex &x) const;
@@ -221,16 +233,15 @@ public:
 	ex simplify_indexed(unsigned options = 0) const;
 	ex simplify_indexed(const scalar_products & sp, unsigned options = 0) const;
 
+	// domains
+	void set_domain(unsigned d);
+
 	// comparison
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
-	bool is_zero() const { 
-#ifndef _MSC_VER
-	  extern const ex _ex0;
-#endif
-	  return is_equal(_ex0); 
-	}
-	bool is_zero_matrix() const;
+	bool is_zero() const;
+        bool is_integer_one() const;
+        bool is_integer_pmone() const;
 	
 	// symmetry
 	ex symmetrize() const;
@@ -244,7 +255,7 @@ public:
 	unsigned return_type() const { return bp->return_type(); }
 	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
 
-	unsigned gethash() const { return bp->gethash(); }
+	long gethash() const { return bp->gethash(); }
 
 private:
 	static ptr<basic> construct_from_basic(const basic & other);
@@ -257,6 +268,11 @@ private:
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
 	void share(const ex & other) const;
+        static ex deep_combine_fractions(ex e);
+        struct combine_map_function : public map_function {
+                ex operator()(const ex & e) override { return deep_combine_fractions(e); }
+        };
+
 
 // member variables
 
@@ -325,53 +341,6 @@ ex::ex(const std::string &s, const ex &l) : bp(construct_from_string_and_lst(s,
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
-inline
-int ex::compare(const ex & other) const
-{
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.total_compares++;
-#endif
-	if (bp == other.bp)  // trivial case: both expressions point to same basic
-		return 0;
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.nontrivial_compares++;
-#endif
-	const int cmpval = bp->compare(*other.bp);
-#if 1
-	if (cmpval == 0) {
-		// Expressions point to different, but equal, trees: conserve
-		// memory and make subsequent compare() operations faster by
-		// making both expressions point to the same tree.
-		share(other);
-	}
-#endif
-	return cmpval;
-}
-
-inline
-bool ex::is_equal(const ex & other) const
-{
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.total_is_equals++;
-#endif
-	if (bp == other.bp)  // trivial case: both expressions point to same basic
-		return true;
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.nontrivial_is_equals++;
-#endif
-	const bool equal = bp->is_equal(*other.bp);
-#if 0
-	if (equal) {
-		// Expressions point to different, but equal, trees: conserve
-		// memory and make subsequent compare() operations faster by
-		// making both expressions point to the same tree.
-		share(other);
-	}
-#endif
-	return equal;
-}
-
-
 // Iterators
 
 class const_iterator : public std::iterator<std::random_access_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
@@ -383,7 +352,7 @@ public:
 	const_iterator() throw() {}
 
 private:
-	const_iterator(const ex &e_, size_t i_) throw() : e(e_), i(i_) {}
+	const_iterator(ex e_, size_t i_) throw() : e(std::move(e_)), i(i_) {}
 
 public:
 	// This should return an ex&, but that would be a reference to a
@@ -395,9 +364,9 @@ public:
 
 	// This should return an ex*, but that would be a pointer to a
 	// temporary value
-	std::auto_ptr<ex> operator->() const
+	std::unique_ptr<ex> operator->() const
 	{
-		return std::auto_ptr<ex>(new ex(operator*()));
+		return std::unique_ptr<ex>(new ex(operator*()));
 	}
 
 	ex operator[](difference_type n) const
@@ -501,7 +470,7 @@ protected:
 namespace internal {
 
 struct _iter_rep {
-	_iter_rep(const ex &e_, size_t i_, size_t i_end_) : e(e_), i(i_), i_end(i_end_) {}
+	_iter_rep(ex e_, size_t i_, size_t i_end_) : e(std::move(e_)), i(i_), i_end(i_end_) {}
 
 	bool operator==(const _iter_rep &other) const throw()
 	{
@@ -759,8 +728,9 @@ inline ex denom(const ex & thisex)
 inline ex numer_denom(const ex & thisex)
 { return thisex.numer_denom(); }
 
-inline ex normal(const ex & thisex, int level=0)
-{ return thisex.normal(level); }
+inline ex normal(const ex & thisex, int level=0, bool noexpand_combined=false,
+                bool noexpand_numer=true)
+{ return thisex.normal(level, noexpand_combined, noexpand_numer); }
 
 inline ex to_rational(const ex & thisex, lst & repl_lst)
 { return thisex.to_rational(repl_lst); }
@@ -780,12 +750,9 @@ inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
 inline ex eval(const ex & thisex, int level = 0)
 { return thisex.eval(level); }
 
-inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=NULL)
+inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=nullptr)
 { return thisex.evalf(level, parent); }
 
-inline ex evalm(const ex & thisex)
-{ return thisex.evalm(); }
-
 inline ex eval_integ(const ex & thisex)
 { return thisex.eval_integ(); }
 
@@ -858,7 +825,7 @@ protected:
 	ex (*ptr)(const ex &);
 public:
 	explicit pointer_to_map_function(ex x(const ex &)) : ptr(x) {}
-	ex operator()(const ex & e) { return ptr(e); }
+	ex operator()(const ex & e) override { return ptr(e); }
 };
 
 template<class T1>
@@ -868,7 +835,7 @@ protected:
 	T1 arg1;
 public:
 	explicit pointer_to_map_function_1arg(ex x(const ex &, T1), T1 a1) : ptr(x), arg1(a1) {}
-	ex operator()(const ex & e) { return ptr(e, arg1); }
+	ex operator()(const ex & e) override { return ptr(e, arg1); }
 };
 
 template<class T1, class T2>
@@ -879,7 +846,7 @@ protected:
 	T2 arg2;
 public:
 	explicit pointer_to_map_function_2args(ex x(const ex &, T1, T2), T1 a1, T2 a2) : ptr(x), arg1(a1), arg2(a2) {}
-	ex operator()(const ex & e) { return ptr(e, arg1, arg2); }
+	ex operator()(const ex & e) override { return ptr(e, arg1, arg2); }
 };
 
 template<class T1, class T2, class T3>
@@ -891,7 +858,7 @@ protected:
 	T3 arg3;
 public:
 	explicit pointer_to_map_function_3args(ex x(const ex &, T1, T2, T3), T1 a1, T2 a2, T3 a3) : ptr(x), arg1(a1), arg2(a2), arg3(a3) {}
-	ex operator()(const ex & e) { return ptr(e, arg1, arg2, arg3); }
+	ex operator()(const ex & e) override { return ptr(e, arg1, arg2, arg3); }
 };
 
 template<class C>
@@ -901,7 +868,7 @@ protected:
 	C &c;
 public:
 	explicit pointer_to_member_to_map_function(ex (C::*member)(const ex &), C &obj) : ptr(member), c(obj) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e); }
 };
 
 template<class C, class T1>
@@ -912,7 +879,7 @@ protected:
 	T1 arg1;
 public:
 	explicit pointer_to_member_to_map_function_1arg(ex (C::*member)(const ex &, T1), C &obj, T1 a1) : ptr(member), c(obj), arg1(a1) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1); }
 };
 
 template<class C, class T1, class T2>
@@ -924,7 +891,7 @@ protected:
 	T2 arg2;
 public:
 	explicit pointer_to_member_to_map_function_2args(ex (C::*member)(const ex&, T1, T2), C &obj, T1 a1, T2 a2) : ptr(member), c(obj), arg1(a1), arg2(a2) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1, arg2); }
 };
 
 template<class C, class T1, class T2, class T3>
@@ -937,7 +904,7 @@ protected:
 	T3 arg3;
 public:
 	explicit pointer_to_member_to_map_function_3args(ex (C::*member)(const ex &, T1, T2, T3), C &obj, T1 a1, T2 a2, T3 a3) : ptr(member), c(obj), arg1(a1), arg2(a2), arg3(a3) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2, arg3); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1, arg2, arg3); }
 };
 
 inline ex ex::map(ex f(const ex &)) const
