@@ -26,12 +26,13 @@ namespace GiNaC {
 
 static ex hermite_evalf(const ex& n, const ex& x, PyObject* parent)
 {
+        // see http://dlmf.nist.gov/18.5.E13
         numeric numn = ex_to<numeric>(n);
         numeric numx = ex_to<numeric>(x);
         std::vector<numeric> numveca, numvecb;
         numveca.push_back(numn / *_num_2_p);
         numveca.push_back(*_num1_2_p + (numn / *_num_2_p));
-        return hypergeometric_pFq(numveca, numvecb, -pow(numx, *_num2_p).inverse(), parent);
+        return pow(numx * (*_num2_p), numn) * hypergeometric_pFq(numveca, numvecb, -pow(numx, *_num2_p).inverse(), parent);
 }
 
 static ex hermite_eval(const ex& n, const ex& x)
@@ -76,8 +77,9 @@ static ex hermite_eval(const ex& n, const ex& x)
 
 static ex hermite_deriv(const ex& n, const ex & x, unsigned deriv_param)
 {
-	GINAC_ASSERT(deriv_param==0);
-	return _ex2 * n * hermite(n-1, x).hold();
+	    if (deriv_param == 0)
+                    throw std::runtime_error("derivative w.r.t. to the index is not supported yet");
+	    return _ex2 * n * hermite(n-1, x).hold();
 }
 
 REGISTER_FUNCTION(hermite, eval_func(hermite_eval).
@@ -91,16 +93,24 @@ REGISTER_FUNCTION(hermite, eval_func(hermite_eval).
 
 static ex gegenb_evalf(const ex& n, const ex &a, const ex& x, PyObject* parent)
 {
+        // see http://dlmf.nist.gov/18.5.E9
         numeric numn = ex_to<numeric>(n);
         numeric numx = ex_to<numeric>(x);
+        numeric numa = ex_to<numeric>(a);
+        numeric num2a = numa * (*_num2_p);
         std::vector<numeric> numveca, numvecb;
-        numveca.push_back(numn / *_num_2_p);
-        numveca.push_back(*_num1_2_p + (numn / *_num_2_p));
-        return hypergeometric_pFq(numveca, numvecb, -pow(numx, *_num2_p).inverse(), parent);
+        numveca.push_back(-numn);
+        numveca.push_back(numn + num2a);
+        numvecb.push_back(numa + (*_num1_2_p));
+        numeric factor = numn + num2a - (*_num1_p);
+        factor = factor.binomial(numn);
+        return factor * hypergeometric_pFq(numveca, numvecb, ((*_num1_p)-numx)/(*_num2_p), parent);
 }
 
 static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
 {
+	numeric numa = 1;
+
 	if (is_exactly_a<numeric>(x)) {
                 numeric numx = ex_to<numeric>(x);
                 if (is_exactly_a<numeric>(n)
@@ -109,23 +119,51 @@ static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
                         return gegenb_evalf(n, a, x, nullptr);
         }
 
-        if (not is_exactly_a<numeric>(n)
-		or not is_exactly_a<numeric>(a))
+        if (not is_exactly_a<numeric>(n))
                 return gegenbauer(n, a, x).hold();
 
         numeric numn = ex_to<numeric>(n);
-	numeric numa = ex_to<numeric>(a);
         if (not numn.info(info_flags::integer) or numn < 0)
                 throw std::runtime_error("gegenb_eval: The index n must be a nonnegative integer");
-        if (not numa.info(info_flags::rational) or numa < 0)
-                throw std::runtime_error("gegenb_eval: The parameter a must be a nonnegative rational");
 
-	// from here on see flint2/fmpq_poly/gegenbauer_c.c
         if (numn.is_zero())
                 return _ex1;
 	if (numn.is_equal(*_num1_p))
 		return _ex2 * a * x;
 
+	if (is_exactly_a<numeric>(a)) {
+		numa = ex_to<numeric>(a);
+		if (numa.is_zero())
+			return _ex0; // C_n^0(x) = 0
+	}
+
+	if (not is_exactly_a<numeric>(a) or numa < 0) {
+		ex p = _ex1;
+		ex sum = _ex0;
+		ex sign = _ex1;
+		ex aa = _ex1;
+		unsigned long nn = numn.to_long();
+
+		sign = - 1;
+		for (unsigned long k=0; k<=nn/2; k++) {
+			sign *= - 1;
+			aa = a;
+			p = 1;
+
+			// rising factorial (a)_{n-k}
+			for (unsigned long i=0; i<nn-k; i++) {
+				p *= aa;
+				aa = aa + 1;
+			}
+
+			p /= numeric(k).factorial();
+			p /= numeric(nn-2*k).factorial();
+			sum += pow(2*x, nn-2*k) * p * sign;
+		}
+		return sum;
+	}
+
+	// from here on see flint2/fmpq_poly/gegenbauer_c.c
 	numeric numer = numa.numer();
 	numeric denom = numa.denom();
 	numeric t = numn.factorial();
@@ -133,9 +171,9 @@ static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
 
 	unsigned long nn = numn.to_long();
 	numeric p = t / (numeric(nn/2).factorial());
-	if (nn%2)
+	if ((nn%2) != 0u)
 		p *= *_num2_p;
-	if (nn&2)
+	if ((nn&2) != 0u)
 		p = -p;
 
 	for (unsigned long k=0; k < nn-nn/2; k++) {
@@ -145,7 +183,7 @@ static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
 
 	p *= denom.power(nn/2);
         ex sum = _ex0;
-	if (nn%2)
+	if ((nn%2) != 0u)
 		sum += x*p;
 	else
 		sum += p;
@@ -164,8 +202,19 @@ static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
         return sum / overall_denom;
 }
 
+static ex gegenb_deriv(const ex& n, const ex & a, const ex & x, unsigned deriv_param)
+{
+	    if (deriv_param == 0)
+                    throw std::runtime_error("derivative w.r.t. to the index is not supported yet");
+	    if (deriv_param == 1)
+                    throw std::runtime_error("derivative w.r.t. to the second index is not supported yet");
+
+	    return _ex2 * a * gegenbauer(n-1, a+1, x).hold();
+}
+
 REGISTER_FUNCTION(gegenbauer, eval_func(gegenb_eval).
                         evalf_func(gegenb_evalf).
+                        derivative_func(gegenb_deriv).
 		        latex_name("C"));
 
 } // namespace GiNaC
